@@ -173,16 +173,6 @@ impl Syntax {
         }
     }
 
-    pub fn static_blocks(self) -> bool {
-        matches!(
-            self,
-            Syntax::Es(EsConfig {
-                static_blocks: true,
-                ..
-            }) | Syntax::Typescript(..)
-        )
-    }
-
     /// Should we parse jsx?
     pub fn jsx(self) -> bool {
         matches!(
@@ -191,14 +181,6 @@ impl Syntax {
         )
     }
 
-    pub const fn optional_chaining(self) -> bool {
-        true
-    }
-
-    pub const fn dynamic_import(self) -> bool {
-        true
-    }
-
     pub fn fn_bind(self) -> bool {
         matches!(self, Syntax::Es(EsConfig { fn_bind: true, .. }))
     }
@@ -328,9 +310,6 @@ pub struct EsConfig {
     #[serde(default)]
     pub import_assertions: bool,
 
-    #[serde(default, rename = "staticBlocks")]
-    pub static_blocks: bool,
-
     #[serde(default, rename = "privateInObject")]
     pub private_in_object: bool,
 
