@@ -565,15 +565,10 @@ impl<I: Tokens> Parser<I> {
 
         self.try_parse_ts(|p| {
             let type_args = p.parse_ts_type_args()?;
-            if is_one_of!(
-                p, ',', '.', '?', ')', ']', ':', '&', '|', '^', '}', "??", "==", "===", "!=",
-                "!==", "&&", "||"
-            ) || is_exact!(p, ';')
-                || eof!(p)
-            {
-                Ok(Some(type_args))
-            } else {
+            if p.is_start_of_expr()? {
                 Ok(None)
+            } else {
+                Ok(Some(type_args))
             }
         })
     }
